//*****************************************************************************
//*****************************************************************************
//  FILENAME:   sensor.c
//  Version: 1.0, 2007/03/08
//  Generated by Andy Kiser
//
//  DESCRIPTION:  sensor specific functions 
//
//-----------------------------------------------------------------------------
// Originally authored by Toradex AG. This work is now available under the terms and conditions
// of the Creative Commons License 'Attribution CC BY', details of which can be found here:
// http://creativecommons.org/licenses/by/3.0/
//*****************************************************************************
//*****************************************************************************
//#define __DEBUG__
#include <m8c.h>        // part specific constants and macros
#include "sensor.h"
#include "psocgpioint.h"
#include "feature.h"
#include "parameters.h"
#include "types_n_macros.h"
#include "lsm303dlm.h"

//----------------------------------------
// Sensor parameters
// Factory Defaults are defined here (NUM_xxx are defined in "sensor.h")
//----------------------------------------
const BYTE FactoryDefault_Param_8   [NUM_PARAM_8] =
{
	0x00,		// report mode
	0x00,		// LED mode
	0x00,		// Acceleration range
	0x00		// Magnetic sensor range
};

const WORD FactoryDefault_Param_16  [NUM_PARAM_16] = 
{
	500,		// report rate [ms]
	500,		// sample rate [ms]
	50000,		// sample rate [us]	(not used for this sensor)
	0			// Words index 3 to 26 for user calibration...
};

const DWORD FactoryDefault_Param_32  [NUM_PARAM_32] =
{
	0x00000000	// currently no 32bit parameters
};

const tParam_168 FactoryDefault_Param_168 [NUM_PARAM_168] =
{
	"Orientation Sensor",		// user device name
	"Frame Number",			// user name Channel 0
	"Acceleration X",		// user name Channel 1
	"Acceleration Y",		// user name Channel 2
	"Acceleration Z",		// user name Channel 3
	"Magnetic Field X",		// user name Channel 4
	"Magnetic Field Y",		// user name Channel 5
	"Magnetic Field Z"		// user name Channel 6
};


//----------------------------------------
// this module's global variables
//----------------------------------------
tReport Report;

//----------------------------------------
// type definitions
//----------------------------------------

typedef enum 
{	
  State_Idle,
  State_Measure_G,
  State_Measure_Mag
} tFSM_Measure;

typedef union 
	{
		int s;
		struct sBigEndian BE;
	} uSigned;


//----------------------------------------
// variables

tFSM_Measure	FSM;
WORD			Countdown_1ms;

BYTE			Acc_Range;
BYTE			Mag_Raw;

BOOL			NewSample = FALSE;

extern BYTE		SOF_Flag; // needed to compensate for FSM processing duration >1ms

// store original drivemodes during suspend
BYTE			DM0_2, DM0_1, DM0_0;
BYTE			DM2_2, DM2_1, DM2_0;



//----------------------------------------
// function prototypes


//-----------------------------------------------------------------------------
// Initializes the Sensor 
//-----------------------------------------------------------------------------
void Sensor_Start ()
{
	LSM303_Start();
	
	FSM =  State_Idle;
}

//-----------------------------------------------------------------------------
// Prepare Sensor for Suspend
//-----------------------------------------------------------------------------
void Sensor_Suspend()
{
	LSM303_Stop();
	
	// store original drivemodes during  suspend
	DM0_2 = PRT0DM2;
	DM0_1 = PRT0DM1;
	DM0_0 = PRT0DM0;
	DM2_2 = PRT2DM2;
	DM2_1 = PRT2DM1;
	DM2_0 = PRT2DM0;
	
}
//-----------------------------------------------------------------------------
// Resume Sensor from Suspend mode
//-----------------------------------------------------------------------------
void Sensor_Resume()
{
	// restore drivemodes for GPIOs to/from the sensor
	PRT2DM2 = DM2_2;
	PRT2DM1 = DM2_1;
	PRT2DM0 = DM2_0;

	PRT0DM2 = DM0_2;
	PRT0DM1 = DM0_1;
	PRT0DM0 = DM0_0;

	Sensor_Start();
}


//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// cannot be handled with the standard functionality in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//
//	return Value: number of Parameter bytes (1-31) that are 
//				  returned in the IN report
//-----------------------------------------------------------------------------
BYTE Sensor_Specific_Feature(void)
{
	BYTE i;

	static const BYTE ValidRegister[] = 
	{
		//--- Magnetic sensor
		0x00, 	
		0x01, 	
		0x02,	
		0x03,	
		0x04,	
		0x05,	
		0x06,	
		0x07, 	
		0x08,	
		0x09,	
		0x0a,	
		0x0b,	
		0x0c,	
		
		//--- Acc sensor
		0x20,
		0x21,
		0x22,
		0x23,
		0x24,
		0x25,
		0x26,
		0x27,
		0x28,
		0x29,
		0x2a,
		0x2b,
		0x2c,
		0x2d,
		0x30,
		0x31,
		0x32,
		0x33,
		0x34,
		0x35,
		0x36,
		0x37
	};
	
	
	if ( (FEATURE_RPT.OUT.Target == eSensor) && (FEATURE_RPT.OUT.ParamSize == 1) )
		for (i=0; i < sizeof(ValidRegister); i++)
			if (FEATURE_RPT.OUT.Index_LSB == ValidRegister[i])
			{
				if (FEATURE_RPT.OUT.Get_nSet)
				{ 
					// GET  -> Read an LSM register
					if (FEATURE_RPT.OUT.Index_LSB < 0x20) // Magnetic sensor?
					{
						FEATURE_RPT.IN.Param[0] = LSM303_RxReg_Mag(FEATURE_RPT.OUT.Index_LSB);
					}
					else
					{
						FEATURE_RPT.IN.Param[0] = LSM303_RxReg_Acc(FEATURE_RPT.OUT.Index_LSB);
					}
					return 1;
				}
				else
				{ 
					// SET  -> Write an LSM register
					if (FEATURE_RPT.OUT.Index_LSB < 0x20) // Magnetic sensor?
					{
						LSM303_TxReg_Mag(FEATURE_RPT.OUT.Index_LSB, FEATURE_RPT.OUT.Param[0]);
					}
					else
					{
						LSM303_TxReg_Acc(FEATURE_RPT.OUT.Index_LSB, FEATURE_RPT.OUT.Param[0]);					
					}
					return 0;
				}
			}	
	return 0;
}

//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// sets / gets a 8/16/32/168bit parameter in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//-----------------------------------------------------------------------------
void Sensor_Specific_Parameter(BYTE *NumberOfReturnedBytes)
{
	// no special parameter handling necessary
}
//-----------------------------------------------------------------------------
// void Sensor_FSM_Measure(void)
//
//	finite state machine to control the measurement process for 
//  the sensor
//
//  IMPORTANT: 	call this routine once every millisecond. Otherwise the timing
//				parameters cannot be met.
//-----------------------------------------------------------------------------
void Sensor_FSM_Measure() 
{
	// decrement millisecond counter
	if (Countdown_1ms != 0)
		Countdown_1ms--;

	// process actual state machine
	switch (FSM)
	{					
		case State_Idle:
			if (Countdown_1ms == 0)
			{
				unsigned char param, temp_range;
				
				// reset samplerate counter
				Get_Param_16(PAR16_SAMPLERATE_MS, VOLATILE, &Countdown_1ms);
				
				// Get acc sensor range...
				Get_Param_8(PAR8_ACC_RANGE, VOLATILE, &temp_range);
				
				// Check if the written value is valid
				// Check if range is changed
				if (temp_range != Acc_Range)
				{
					if ((temp_range <= 1) || (temp_range == 3))
						{
						unsigned char b = 0x80;
						Acc_Range = temp_range;
						b = 0x80 | ((temp_range & 0x03) << 4);
						LSM303_TxReg_Acc(0x23, b);
					}
				}
				
				// Get magnetic sensor param
				Get_Param_8(PAR8_MAG_RANGE, VOLATILE, &Mag_Raw);

				FSM = State_Measure_G;		
			}
			break;			
		
		
		case State_Measure_G:
			
			if (LSM303_Check_DRDY_Acc())
			{
				uSigned acc_X, acc_Y, acc_Z;				 				
				unsigned char factor = 16;
				// Read Acc...
				// Can be optimized later with multi-read...
				acc_X.BE.Lo = LSM303_RxReg_Acc(0x28);				
				acc_X.BE.Hi = LSM303_RxReg_Acc(0x29);

				acc_Y.BE.Lo = LSM303_RxReg_Acc(0x2A);				
				acc_Y.BE.Hi = LSM303_RxReg_Acc(0x2B);

				acc_Z.BE.Lo = LSM303_RxReg_Acc(0x2C);				
				acc_Z.BE.Hi = LSM303_RxReg_Acc(0x2D);
				
				// Format Acc...
				// Value signed on 12bits
				// Scale +-2g -> 1mg/LSB
				//       +-4g -> 2mg/LSB
				//       +-8g -> 4mg/LSB
				// Error smaller than sensitivity error!!! -> ok...
				factor /= (Acc_Range+1); 
				acc_X.s /= factor;
				acc_Y.s /= factor;
				acc_Z.s /= factor;
				
				
				// fill measured data into IN REPORT
				Report.Ch1_LSB = acc_X.BE.Lo;
				Report.Ch1_MSB = acc_X.BE.Hi;
				Report.Ch2_LSB = acc_Y.BE.Lo;
				Report.Ch2_MSB = acc_Y.BE.Hi;
				Report.Ch3_LSB = acc_Z.BE.Lo;
				Report.Ch3_MSB = acc_Z.BE.Hi;				
				
				FSM = State_Measure_Mag;
			}
		break;
		
		case State_Measure_Mag: 
			
			// NOTE: The sync is done only on the G sensor since the magnetic sensor is anyway much slower... 
			
			// Check Data Ready input flag before reading values from sensor		
			if (LSM303_Check_DRDY_Mag())	
				{
				uSigned mag_X, mag_Y, mag_Z;
				
				// Read Mag...
				
				mag_X.BE.Hi = LSM303_RxReg_Mag(0x03);
				mag_X.BE.Lo = LSM303_RxReg_Mag(0x04);

				mag_Y.BE.Hi = LSM303_RxReg_Mag(0x07);   // LSM303LDLH was 0x05
				mag_Y.BE.Lo = LSM303_RxReg_Mag(0x08);   // LSM303LDLH was 0x06
					
				mag_Z.BE.Hi = LSM303_RxReg_Mag(0x05);   // LSM303LDLH was 0x07 
				mag_Z.BE.Lo = LSM303_RxReg_Mag(0x06);   // LSM303LDLH was 0x08 

				// Format Mag values...
				if (!Mag_Raw)
					{
						mag_X.s = MagSensorToPhysicalVal(mag_X.s, 0);
						mag_Y.s = MagSensorToPhysicalVal(mag_Y.s, 0);
						mag_Z.s = MagSensorToPhysicalVal(mag_Z.s, 1);
					}
				
				// fill measured data into IN REPORT
				Report.Ch4_LSB = mag_X.BE.Lo;
				Report.Ch4_MSB = mag_X.BE.Hi;
				
				Report.Ch5_LSB = mag_Y.BE.Lo;
				Report.Ch5_MSB = mag_Y.BE.Hi;
				
				Report.Ch6_LSB = mag_Z.BE.Lo;
				Report.Ch6_MSB = mag_Z.BE.Hi;								
				}
				
				NewSample=TRUE;				
				
				// Update State Machine
				FSM = State_Idle;									
		break;
	}
	
	// compensate timing if processing above took longer than 1ms
	while (SOF_Flag >0)
	{
		SOF_Flag--;
		if (Countdown_1ms != 0)
			Countdown_1ms--;
	}
}

//-----------------------------------------------------------------------------
void Sensor_Debug(void)
{
/* currently no debug function */
}